<?php

/**
 * Remote Backend API calls
 * 
 * Provide common code to be used across different platforms/frameworks
 * Base class for calls to backend
 * 
 * @category   Remsto
 * @package    Enquir3_Library
 * @author     Ian Burnup
 * @copyright  Irun 2019
 */

if (class_exists('RemstoClient')) {
  return;
}


class RemstoClient {

  protected $config  = array();
  protected $user_id = 0;

  public function __construct($client_conf, $user_id = 0) {
    $this->config  = $client_conf;
    $this->user_id = $user_id;
  }

  /**
   * Get security token for this request.
   * @return array security required for api call.
   */
  private function getSecurityToken() {
    $time = time() + 5 * 60;

    $client_key = $this->config['client_key'];
    $client_id = $this->config['client_id'];

    $hash = hash_hmac('sha256', $time, $client_key);

    $security_token = array(
      'time' => $time,
      'client_id' => $client_id,
      'hash' => $hash,
    );

    return $security_token;
  }

  /**
   * Send api call to remsto server
   * @param string $end_point Full url (excluding server) of api end point e.g. /api/v1/news/list
   * @param array $data post request data (this will be sent as a serialised string)
   * @return array (or object) response from remsto server.
   * @throws Exception
   */
  public function sendData($end_point, $data = null) {

    $post_data = $this->getSecurityToken();
    $post_data['user_id']=$this->user_id;

    $url = $this->config['remote_url'] . $end_point;

    if ($data) {
      $post_data['data'] = serialize($data);
    }
    
    $req = curl_init();
    
    if(isset($data['image_file'] )) {
      $mime_type=image_type_to_mime_type(exif_imagetype($data['image_file']));
      $post_data['image_file'] = new CURLFile($data['image_file'], $mime_type, $data['image_file']);
      $post_data['image_name'] = $data['image_name'];
      curl_setopt($req, CURLOPT_HTTPHEADER, array("Content-Type:multipart/form-data"));
    }
	
    /**  MULTI-FILE UPLOAD  **/
    /****************************************************************************/
    # This should allow multiple files, along with files that belong to the same field.
    # Example stucture:
    # $data['_upload_files'] = array(
    #   'my_upload_0' => array('field' => 'my_image_field_src', 'type' => 'image', 'file' => 'my_file.jpg', 'name' => 'my image file')
    # );
			
    if (isset($data['_upload_files'])) {
      foreach ($data['_upload_files'] as $key => $file) {
        $mime_type=image_type_to_mime_type(exif_imagetype($file['file']));
        $post_data[$key] = new CURLFile($file['file'], $mime_type, $data['image_file']);
      }
      curl_setopt($req, CURLOPT_HTTPHEADER, array("Content-Type:multipart/form-data"));
    }
    /****************************************************************************/

    curl_setopt($req, CURLOPT_URL, $url);
    curl_setopt($req, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($req, CURLOPT_POST, true);
    curl_setopt($req, CURLOPT_POSTFIELDS, $post_data);
	
	// Temp solution to allow database site with SSL: http://unitstep.net/blog/2009/05/05/using-curl-in-php-to-access-https-ssltls-protected-sites/
	curl_setopt($req, CURLOPT_SSL_VERIFYPEER, false);

    $response = curl_exec($req);
    $response_data = curl_getinfo($req);

    curl_close($req);
    // Check data sent and received okay.
    $http_code = $response_data['http_code'];
    if ($http_code !== 200) {
      throw new Exception("API Request Sending Error HTTP Code: $http_code " . $response_data['url'], 3002);
    }
    $res = json_decode($response);
    if ($res) {
      if ($res->status !== 0) {
        throw new Exception('API Request Error: ' . $res->msg, 3001);
      }
    }
    else {
      throw new Exception('Could not parse response: ' . $response, 3003);
    }

    return $res;
  }
  
  /**
   * Get taxonomy terms for given taxonomy id.
   * @param int $tid taxonomy id
   * @return array taxonomy terms.
   */
  public function getTaxonomyTerms($tid) {

    try {

      $post_data = array();
      
      $data = $this->sendData("/api/v1/taxonomy_terms/get/$tid", $post_data);
      return $data->terms;
    }
    catch (Exception $ex) {
      print ('<!-- Debug ' . $ex->getMessage() . ' -->');
    }
  }
  
  /**
   * Get taxonomy with all its terms
   * @param type $tid
   * @return type
   */
  public function getTaxonomy($tid, $parent_type = 0, $parent_id = 0) {

    try {

      $post_data = array(
        'parent_type' => $parent_type,
        'parent_id' => $parent_id,
      );
      
      $data = $this->sendData("/api/v1/taxonomy/get/$tid", $post_data);
      return $data->taxonomy;
    }
    catch (Exception $ex) {
      print ('<!-- DEBUG:' . $ex->getMessage() . '-->');
    }
  }

  /**
   * Get all possible taxonomy vocabularies
   * @return array Taxonomy terms.
   */
  public function listVocabularies() {
    try {
      $data = $this->sendData(
              '/api/v1/taxonomy/get',
              array());
      return $data->taxonomies;
    }
    catch (Exception $ex) {
      print ('<!-- Debug ' . $ex->getMessage() . ' -->');
      throw $ex;
    }
  }
  
  /**
   * Add taxonomy
   * @param type $taxonomy
   * @return type
   */
  public function addTaxonomy($taxonomy) {
    try {
      $taxonomy['type'] = 1; //local taxonaomy
      $post_data = array(
        'taxonomy' => $taxonomy,
        'parent_type' => 0,
        'parent_id' => 0,      
      );

      $data = $this->sendData("/api/v1/taxonomy/add", $post_data);
      return $data->tid;
    }
    catch (Exception $ex) {
      print ('<!-- Debug ' . $ex->getMessage() . ' -->');
    }
  }

  /**
   * Get a list of taxonomy terms by taxonomy title.
   * @param string $title taxonomy term title
   * @param int $tid id of vocabulary
   * @param int $parent_term_id term id of parent term
   * @return array taxonomy terms.
   */
  public function getTaxonomyTermsByTitle($title, $tid=null, $parent_term_id=null, $like=false) {
    try {

      $post_data = array(
          'title'=>$title,
          'tid'=>$tid,
          'parent_term_id'=>$parent_term_id,
          'like'=>$like,
      );

      $data = $this->sendData("/api/v1/taxonomy/terms/get/title", $post_data);
      return $data->terms;
    }
    catch (Exception $ex) {
      print ('<!-- Debug ' . $ex->getMessage() . ' -->');
    }
  }

  /**
   * Create local taxonomy term
   * @param type $tid
   * @param type $term
   * @return type
   */
  public function createTaxonomyTerm($tid, $term) {
    try {

      $term['type'] = 1; //local taxonaomy term
      $post_data = array(
        'term' => $term,
        'parent_type' => 0,
        'parent_id' => 0, 
      );

      $data = $this->sendData("/api/v1/taxonomy/$tid/post", $post_data);
      return $data->term;
    }
    catch (Exception $ex) {
      print ('<!-- Debug ' . $ex->getMessage() . ' -->');
    }
  }

  /**
   * Delete specified taxonomy term
   * @param int $term_id term id to delete
   * @return type
   */
  public function deleteTaxonomyTerm($tid, $term_id) {

    try {

      $post_data = array(
        'term_id' => $term_id,
        'tid' => $tid,
      );

      $data = $this->sendData("/api/v1/taxonomy/term/delete/$term_id", $post_data);
      return $data->term;
      
    }
    catch (Exception $ex) {
      print ('<!-- Debug ' . $ex->getMessage() . ' -->');
    }
  }

  public function getTaxonomyTermById($term_id) {
    try {
      $data = $this->sendData("/api/v1/taxonomy/term/get/$term_id", array());
      return $data->term;
    }
    catch (Exception $ex) {
      print ('<!-- Debug ' . $ex->getMessage() . ' -->');
    }
  }
  
  
  
  
  
	public function userAccess($parent_type, $parent_id, $item_type, $item_id, $user_id, $access) {

		static $permissions = array();

		#$site_id = $this->config['client_id'];
		#$user_id = $this->user_id;
		#IrunUtil::debug("[userAccess]: {$user_id} / {$access}");
		
		if (!isset($permissions[$user_id])) {
			$permissions[$user_id] = array();
			
			try {
			  $data = $this->sendData('/api/v1/site/get-user-permissions');
			  
			  if (!empty($data->user_access)) {
				  $user_access = json_decode(json_encode($data->user_access), true);
				  $permissions[$user_id] = $user_access;
			  }

			} catch (Exception $e) {
				#print ($e->getMessage());
				return FALSE;
			}
		}
		
		// If we have no permissions for this user, return FALSE.
		if (empty($permissions[$user_id])) {
			return FALSE;
		}
		
		#IrunUtil::debug(func_get_args());
		#IrunUtil::debug($permissions);
				
		foreach ($permissions[$user_id] as $permission) {
			
			if ($permission['parent_type'] != $parent_type) { continue; }
			if (($permission['parent_id'] != 0) && ($permission['parent_id'] != $parent_id)) { continue; }
			
			if ($permission['item_type'] != $item_type) { continue; }
			if (($permission['item_id'] != 0) && ($permission['item_id'] != $item_id)) { continue; }
			
			/**
			 * Bitwise OR of:
			 *  1 - view
			 *  2 - edit
			 *  4 - create
			 *  8 - delete
			 */
			switch ($access) {
				case 'view'					: $access_bit = 1; break;
				case 'edit'					: $access_bit = 2; break;
				case 'create'				: $access_bit = 4; break;
				case 'delete'				: $access_bit = 8; break;
				case 'permanently_delete'	: $access_bit = 16; break;
				default						: $access_bit = 0; break;
			}

			#$result = ($permission['user_access'] & $access_bit) ? 'YES' : 'NO' ;
			#IrunUtil::debug($permission);
			#IrunUtil::debug('[USER CHECK]: ' . $access . ' = ' . $result);

			return ($permission['user_access'] & $access_bit) ? TRUE : FALSE ;
		}

		return FALSE;
	}
	

	public function linkSiteToParent($parent_type, $parent_id, $old_parent_type, $old_parent_id, $site_id = FALSE) {
		
		$post_data = array(
			 'parent_type'		=> $parent_type,
			 'parent_id'		=> $parent_id,
			 'old_parent_type'	=> $old_parent_type,
			 'old_parent_id'	=> $old_parent_id,
		);
		
		if (!empty($site_id)) {
			$post_data['site_id'] = $site_id;
		}

		try {
			$data = $this->sendData("/api/v1/site/link", $post_data);
			return $data->term;
		} catch (Exception $ex) {
			print ('<!-- Debug ' . $ex->getMessage() . ' -->');
		}
	}
	

}
